/*

    Copyright ( C ) 2003 Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation;
    version 2 of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    */

#include "kstereovolumecontrolgui_impl.h"

#include <qframe.h>
#include <kdebug.h>
#include <iostream>

using namespace Arts;

KStereoVolumeControlGui_impl::KStereoVolumeControlGui_impl( QFrame* w ) : KLayoutBox_impl( w ? w : new QFrame( 0 ) )
{
kdDebug()<<"KStereoVolumeControlGui_impl::KStereoVolumeControlGui_impl( QFrame* "<<w<<" )"<<endl;
	_qframe = dynamic_cast<QFrame*>( _qwidget );
	_mapper = new KStereoVolumeControlGui_EventMapper( this, _qframe );
	this->addWidget( _left );
	this->addWidget( _tickmarks );
	_tickmarks.hSizePolicy( spMaximum );
	this->addWidget( _right );
	_tickmarks.min( _left.mindB() );
	_tickmarks.max( 0 );
	_tickmarks.minstep( 1 );
	//_left.volume( true );
	//_right.volume( true );
kdDebug()<<"Arts::posLeft="<<Arts::posLeft<<" Arts::posRight="<<Arts::posRight<<endl;
	_tickmarks.position( posLeft|posRight );
	this->layoutmargin( 1 ); this->linewidth( 1 ); this->frameshape( Arts::Box ); this->frameshadow( Arts::Plain );
//kdDebug()<<"KStereoVolumeControlGui_impl::KStereoVolumeControlGui_impl() done."<<endl;
}

void KStereoVolumeControlGui_impl::constructor( Arts::StereoVolumeControl svc ) {
kdDebug()<<"KStereoVolumeControlGui_impl::constructor( Arts::StereoVolumeControl svc"<</*!svc.isNull()<<*/" )"<<endl;
	_svc = svc;
	connect( svc, "currentVolumeLeft_changed", _left, "invalue" );
	connect( svc, "currentVolumeRight_changed", _right, "invalue" );

	_left.volumevalue( svc.scaleFactor() );
	_right.volumevalue( svc.scaleFactor() );

	connect( svc, "scaleFactor_changed", _left, "volumevalue" );
	connect( _left, "volumevalue_changed", svc, "scaleFactor" );
	connect( svc, "scaleFactor_changed", _right, "volumevalue" );
	connect( _right, "volumevalue_changed", svc, "scaleFactor" );

_mapper->_timer->start( 100 );
}

void KStereoVolumeControlGui_impl::updateValues() {
	_left.invalue( _svc.currentVolumeLeft() );
	_right.invalue( _svc.currentVolumeRight() );
}

REGISTER_IMPLEMENTATION( KStereoVolumeControlGui_impl );

// vim: sw=4 ts=4
#include "kstereovolumecontrolgui_impl.moc"

