/***************************************************************************
                           exportwave.cpp  -  description
                              -------------------
    begin                : Fir Mar 28 2003
    copyright            : ( C ) 2002 by Arnold Krille
    email                : arnold@arnoldarts.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   ( at your option ) any later version.                                 *
 *                                                                         *
 ***************************************************************************/

#include "exportwave.h"

#include "globals.h"
#include "rawreader.h"
#include "wavewriter.h"

#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <klocale.h>

#include <kartsserver.h> // somehow needed for malloc...

/* Porting the Wave-export from Nikolas Zimmermann into an extra file... */

void exportWave( RecordFile* file ) {

	if( ! file )
	{
		KMessageBox::error( 0, i18n("You did not record anything yet!") );
		return;
	}

	QFileInfo fileInfo( file->file() );
	KURL input( fileInfo.filePath() );
	
	KURL output = KFileDialog::getSaveURL( fileInfo.dirPath(), "*.wav", 0, "Export as WAVE" );
	
	if( !output.isValid() || !input.isValid() )
	{
		KMessageBox::error( 0, i18n("Error while preparing wave exporting!") );
		return;
	}
	
	bool ok;

	// 1. Initialize RawReader
	RawReader *rawReader = new RawReader();
	ok = rawReader->setup(input, ReaderWriter::READ);

	if(!ok)
	{
		KMessageBox::error( 0, i18n("Unable to read input file!") );
		delete rawReader;
		return;
	}

	// 2. Initialize WaveWriter
	WaveWriter *waveWriter = new WaveWriter();
	ok = waveWriter->setup( output, ReaderWriter::WRITE );

	if(!ok)
	{
		KMessageBox::error( 0, i18n("Couldn't initialize wave writing!") );
		delete rawReader;
		delete waveWriter;
		return;
	}


	// 3. Write wave header
	ok = waveWriter->writeWaveHeader( false );

	if(!ok)
	{
		KMessageBox::error( 0, i18n("Failed writing wave header!") );
		return;
	}

	// 4. Prepare raw reading
	Q_INT32 *inputBuffer = (Q_INT32 *) malloc( GLOBAL_BUFFERSIZE * sizeof(Q_INT32) );
	if( !inputBuffer )
	{
		KMessageBox::error( 0, i18n("Couldn't allocate memory for the input buffer!") );
		delete rawReader;
		delete waveWriter;
		return;
	}

	// 5. Read raw input file and
	//    pipe input data to the WaveWriter
	Q_INT32 read = 0, wrote = 0, blockSize = 0;

	do {
		blockSize = rawReader->readFile(inputBuffer, GLOBAL_BUFFERSIZE);
		wrote += waveWriter->writeWaveData(inputBuffer, blockSize);
		read += blockSize;
	} while(!rawReader->eof());

	// 6. Statistics
	kdDebug() << "Read  " << (read * GLOBAL_CHANNELS) << " bytes from raw input source." << endl;
	kdDebug() << "Wrote " << (wrote * GLOBAL_CHANNELS) + 44 << " bytes as MS PCM wave file." << endl;

	// 7. Finish wave file
	ok = waveWriter->finishFile();

	if( !ok )
		KMessageBox::error( 0, i18n("Couldn't finish wave file!") );
}

