#include "vequalizer.h"
#define EQVIEW_CPP
#include "equalizerview.h"
#undef EQVIEW_CPP
#include "app.h"

#include <knuminput.h>
#include <kdialog.h>
#include <kiconloader.h>
#include <klocale.h>

#include <qlayout.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qfileinfo.h>
#include <qhbox.h>
#include <qvbox.h>

#define EQ (napp->vequalizer())



////////////////////////////////////////////////
// PresetList

PresetList::PresetList(QWidget *parent)
	: KListView(parent)
{
	setItemsRenameable(true);
	setRenameable(0, true);
	addColumn("");
	// create another column to store cruft in
	addColumn("", 0);
	header()->hide();
}

void PresetList::rename(QListViewItem *item, int c)
{
	// We can't rename the "Custom" metapreset
	if (item->text(0)==i18n("Custom"))
		return;

	// Or presets we don't have write access to
	if (!QFileInfo(item->text(1)).isWritable()) 
		return;

	KListView::rename(item, c);
}






EqualizerLevel::EqualizerLevel(QWidget *parent, VBand band)
	: QWidget(parent), mBand(band)
{
	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->setAutoAdd(true);

	mSlider = new QSlider(-200, 200, 50, 0, Qt::Vertical, this);
	connect(mSlider, SIGNAL(valueChanged(int)), SLOT(changed(int)));
	mLabel = new QLabel("", this);
	mLabel->setAlignment(AlignHCenter | AlignVCenter);

	setMinimumHeight(150);
//	setMinimumWidth(30);
	setMinimumWidth(kapp->fontMetrics().width("158kHz"));
	
	setBand(band);
	
	connect(EQ, SIGNAL(modified()), SLOT(changed()));
	connect(mSlider, SIGNAL(valueChanged(int)), SLOT(changed(int)));
}

void EqualizerLevel::setBand(VBand band)
{
	mBand = band;
	mLabel->setText(band.format());
	changed();
}

void EqualizerLevel::changed()
{
	mSlider->setValue(-mBand.level());
}

void EqualizerLevel::changed(int v)
{
	mBand.setLevel(-v);
}


///////////////////////////////////////////////
// EqualizerView

EqualizerView::EqualizerView()
	: KDialogBase((QWidget*)0L, 0, false, i18n("Equalizer"), Help | Close, Close, true),
	  first(true), mGoingPreset(false)
{
	mBands.setAutoDelete(true);
}

void EqualizerView::show()
{
	if (first)
	{
		first=false;
		setIcon(SmallIcon("noatun"));
		
		QFrame *frame = makeMainWidget();

		QGridLayout *grid = new QGridLayout(frame);
		
		{ // preamp slider
			QWidget *preamp=new QWidget(frame);
			preamp->setMinimumWidth(kapp->fontMetrics().width("158kHz"));
			(new QVBoxLayout(preamp))->setAutoAdd(frame);
			mPreamp = new QSlider(-200, 200, 25, 0, Qt::Vertical, preamp);
			QLabel *lpre = new QLabel("+/-", preamp);
			lpre->setAlignment(AlignHCenter | AlignVCenter);
			grid->addWidget(preamp, 0, 0);
			connect(mPreamp, SIGNAL(valueChanged(int)), SLOT(setPreamp(int)));
			connect(EQ, SIGNAL(preampChanged(int)), SLOT(changedPreamp(int)));
		}
		
		{ // number of bands
			mBandCount = new KIntNumInput(frame);
			mBandCount->setRange(EQ->minBands(), EQ->maxBands());
			mBandCount->setLabel(i18n("&Number of Bands"));
			grid->addWidget(mBandCount, 1, 1);
			connect(mBandCount, SIGNAL(valueChanged(int)), EQ, SLOT(setBands(int)));
		}
		
		{ // editor tabs
			QTabWidget *tabs = new QTabWidget(frame);
			grid->addWidget(tabs, 0, 1);
			
			{ // slider mode
				mSliders = new QFrame(tabs);
				tabs->addTab(mSliders, i18n("Sliders"));
				(new QHBoxLayout(mSliders))->setAutoAdd(true);
			}
		}
		
		{ // presets
			QVBoxLayout *layouthere = new QVBoxLayout(frame);
			grid->addLayout(layouthere, 0, 2);
			QWidget *widget=new QLabel(i18n("<b>Presets</b>"), frame);
			layouthere->addWidget(widget);

			mPresets=new PresetList(frame);
			layouthere->addWidget(mPresets);

			mRemovePreset=new QPushButton(i18n("&Remove Preset"), frame);
			mRemovePreset->setEnabled(false);
			layouthere->addWidget(mRemovePreset);
			connect(mRemovePreset, SIGNAL(clicked()), SLOT(remove()));

			mAddPreset=new QPushButton(i18n("&Add Preset"), frame);
			layouthere->addWidget(mAddPreset);
			connect(mAddPreset, SIGNAL(clicked()), SLOT(create()));

			new QListViewItem(mPresets, i18n("Custom"));

			connect(
					mPresets,
					SIGNAL(currentChanged(QListViewItem*)),
					SLOT(select(QListViewItem*))
				);
			connect(
					mPresets,
					SIGNAL(itemRenamed(QListViewItem*)),
					SLOT(rename(QListViewItem*))
				);
			
			// populate the preset list
			QValueList<VPreset> presets = EQ->presets();
			for (
					QValueList<VPreset>::Iterator i=presets.begin();
					i != presets.end(); ++i
				)
			{
				created(*i);
			}
			
			connect(EQ, SIGNAL(created(VPreset)), SLOT(created(VPreset)));
			connect(EQ, SIGNAL(renamed(VPreset)), SLOT(renamed(VPreset)));
			connect(EQ, SIGNAL(removed(VPreset)), SLOT(removed(VPreset)));
		}
		
		{ // enabled checkbox
			mOn = new QCheckBox(i18n("&Enabled"), frame);
			grid->addWidget(mOn, 1, 2);
			mOn->setChecked(EQ->isEnabled());
			connect(mOn, SIGNAL(toggled(bool)), EQ, SLOT(setEnabled(bool)));
			connect(EQ, SIGNAL(enabled(bool)), mOn, SLOT(setChecked(bool)));
		}
		
		connect(EQ, SIGNAL(changed()), SLOT(changedEq()));
		connect(EQ, SIGNAL(changedBands()), SLOT(changedBands()));
		
		changedBands();
		changedEq();
	}
	KDialogBase::show();
}

QListViewItem *EqualizerView::itemFor(const QString &filename)
{
	for (QListViewItem *i=mPresets->firstChild(); i!=0; i=i->itemBelow())
	{
		QString t = i->text(1);
		if ((t.length()==0 && filename.length()==0) || t==filename)
			return i;
	}
	return 0;
}

QListViewItem *EqualizerView::itemFor(const VPreset &preset)
{
	return itemFor(preset.file());
}

// why is it that when you move a QSlider up, it goes down?
void EqualizerView::setPreamp(int x)
{
	EQ->setPreamp(-x);
}

void EqualizerView::changedPreamp(int x)
{
	mPreamp->setValue(-x);
}


void EqualizerView::changedBands()
{
	mBands.clear();
	
	VEqualizer &eq = *EQ;
	for (int i=0; i < eq.bands(); ++i)
	{
		EqualizerLevel *l = new EqualizerLevel(mSliders, eq[i]);
		l->show();
		mBands.append(l);
	}

	mBandCount->setValue(eq.bands());
	changedEq();
}

void EqualizerView::changedEq()
{
	if (!mGoingPreset)
	{
		QListViewItem *customitem = itemFor("");
		if (!customitem) // this should never happen!
			return;
		mPresets->setSelected(customitem, true);
	}
}

void EqualizerView::removed(VPreset p)
{
	delete itemFor(p);
}

void EqualizerView::created(VPreset p)
{
	// store the filename in QListViewItem::text(1)
	QString n = p.name();
	QString f = p.file();
	new QListViewItem(mPresets, n, f);
}

void EqualizerView::renamed(VPreset p)
{
	QListViewItem *renamed = itemFor(p);
	if (!renamed) // WTF !
	{
		created(p);
		return;
	}
	renamed->setText(0, p.name());
}

void EqualizerView::remove()
{
	QListViewItem *current=mPresets->currentItem();
	if (current->text(0)==i18n("Custom"))
		return;
	QListViewItem *then=current->itemAbove();
	if (!then) then=current->itemBelow();

	if (then)
		mPresets->setSelected(then, true);
	
	VPreset p = EQ->presetByFile(current->text(1));
	p.remove();
}

void EqualizerView::create()
{
	VPreset p = EQ->createPreset(i18n("New Preset"));

	mGoingPreset = true;

	// Load the new preset
	p.load();

	// We should have just made a list view item for this preset
	// See EquilizerView::presetAdded()
	QListViewItem *i = itemFor(p);

	if (i) 
		mPresets->setSelected(i, true);

	mGoingPreset = false;
}

void EqualizerView::rename(QListViewItem *item)
{
	EQ->presetByFile(item->text(1)).setName(item->text(0));
	item->setText(0, EQ->presetByFile(item->text(1)).name());
}

void EqualizerView::select(QListViewItem *item)
{
	mGoingPreset = true;
	EQ->presetByFile(item->text(1)).load();
	mGoingPreset = false;
	mRemovePreset->setEnabled(item->text(1).length());
}

#undef EQ

#include "equalizerview.moc"
